#define USE_ARDUINO_INTERRUPTS true
#include <Wire.h>
#include <hd44780.h>
#include <PulseSensorPlayground.h>
#include <hd44780ioClass/hd44780_I2Cexp.h> 

#define BUTTON PD7
#define RED 11
#define GREEN 10
#define BLUE 9

#define ENABLE 0
#define INITIALIZING 1
#define MEASURE 2
#define FINISHED 3

hd44780_I2Cexp lcd;
PulseSensorPlayground pulseSensor;

const int PULSE_INPUT = A0;
const int THRESHOLD = 550;

const int buzzerPin = 6;
const int Button = 2; //Button used for reset
   
int presses = 0;
int ts = 0, BPM, lastBPM;

int i, r = 0, g = 0, b = 0;
int state = ENABLE, lastState = ENABLE;
int bpms = 0, no_bpms = 0;

bool clearLCD = true;
volatile short countdown = 3;
volatile short counter = 60;

void pin_ISR()
{
  countdown = 15;
  state = INITIALIZING;
}

void lightRGB (int red, int green, int blue) {
    analogWrite(RED, red);
    analogWrite(GREEN, green);
    analogWrite(BLUE, blue);
}

void timerInit()
{
 cli();
  TCCR0A = 0;
  TCCR0B = 0;
  TCNT0  = 0; //initialize counter value to 0
 
  OCR0A = 255;
  TCCR0A |= (1 << WGM01); // mod CTC
  TCCR0B |= (1 << CS02) | (1 << CS00); // 1024 prescaler
  TIMSK0 |= (1 << OCIE0A);
  sei(); 
}

void setup()
{
  Serial.begin(9600);
  lcd.begin(16, 2);
  
  pinMode(Button, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(Button), pin_ISR, FALLING);

  pinMode(BUTTON, INPUT_PULLUP);
  
  // RGB setup
  pinMode(RED, OUTPUT);
  pinMode(GREEN, OUTPUT);
  pinMode(BLUE, OUTPUT);

  PCICR |= (1 << PCIE2); // enable the pin change interrupt
  PCMSK2 |= (1 << PCINT23); // Turns on PCINT23 (PD7)

  pulseSensor.analogInput(PULSE_INPUT); 
  pulseSensor.setThreshold(THRESHOLD);   

  if (!pulseSensor.begin()) {
    Serial.println("FAILED");
  } else {
    Serial.println("SUCCESS");
  }

  timerInit();
}
// Decrease counter
void count_sec() {
   if (counter == 0)
    {
      counter = 60;
      if (countdown != 0)
      {
        countdown -= 1;
      }
    }
    else
    {
      counter--;
    }
}
ISR(TIMER0_COMPA_vect)
{ 
  //Count aprox. 3s(countdown) before moving to next state
  if (state == ENABLE) {
     if (countdown == 0) {
      state = INITIALIZING;
      lastState = ENABLE;
      countdown = 15;
    }
    count_sec();
  } 
    //Measure heart rate for aprox. 15s(countdown) before 
   // displaying  final resulting BPM
   if (state == MEASURE) {
     if (countdown == 0) {
      state = FINISHED;
      lastState = MEASURE;
    }
    count_sec();
  } 
 
}

ISR(PCINT2_vect)
{
if ((PIND & (1 << PD7)) == 0 && state == INITIALIZING){
    if (millis() - ts > 300) {
      ts = millis();
      presses++;
    }    
  }
}


void loop()
{ 
  if (state == ENABLE) {
     lcd.setCursor(0, 1);
     lcd.print("    WELCOME!   "); 
  }
  if (state == INITIALIZING) {  

     //Display indications for user
      if (presses == 0) {
          if (clearLCD) {
             clearLCD = false;
             lcd.clear();
          }   
          lcd.setCursor(0, 0);
          lcd.print("Press button to"); 
          lcd.setCursor(0, 1);
          lcd.print("start measuring");
     
     //Button PD7 is pressed 
     } else {
          state = MEASURE;
          countdown = 15;
          lastState = INITIALIZING;
          presses = 0;
     }
  }
  if (state == MEASURE) {
      if (lastState != state) {
          lcd.clear();
          lastState = MEASURE;
      }
      BPM = pulseSensor.getBeatsPerMinute();  
      
      if (pulseSensor.sawStartOfBeat()) { // Constantly test to see if "a beat happened". 

       no_bpms++;
       bpms += BPM;
       
       lcd.clear();
       lcd.setCursor(0, 0);
       lcd.print("BPM: ");   
       lcd.setCursor(0, 1);
       
       lcd.print(BPM);
       Serial.println(BPM);
       tone(buzzerPin, 1000, 10);

      //Set color for RGB considering BPM value
       if (BPM < 50) {
          b = 250;
          g = 150;
          r = 100;
        } else if (BPM < 100) {
          r = b = 255;
          g = 0;
        } else if (BPM < 120) {
          b = 100;
          g = 150;
          r = 0;
        } else {
          g = b = 255;
          r = 0;
        }   
       lightRGB(r, g, b);
      }
  } 

  if (state == FINISHED) {
      if (lastState != state) {
          lcd.clear();
          lastState = FINISHED;
          BPM = bpms / no_bpms;
      }
     //Print result
     lcd.setCursor(0, 0);
     lcd.print("BPM: "); 
     lcd.setCursor(5, 0);
     lcd.print(BPM);
     lcd.setCursor(0, 1);
     
       if (BPM < 50) {
         lcd.print("Slow heart rate!");
       } else if (BPM < 100) {
         lcd.print("Good heart rate!");
        } else if (BPM < 130) {
         lcd.print("High heart rate!");
        } else {
         lcd.print("Tahicardie");
        }   
     
     bpms = no_bpms = 0;
     
  }
  delay(20);     
  
}
